      /*
       *  multi_cpu.cpp
       *
       *  Copyright (c) 2007 by Kuscsik Peter <kuscsikp@gmail.com>
       *
       *  This program is free software; you can redistribute it and/or modify
       *  it under the terms of the GNU General Public License as published by
       *  the Free Software Foundation; either version 2 of the License, or
       *  (at your option) any later version.
       *
       *  This program is distributed in the hope that it will be useful,
       *  but WITHOUT ANY WARRANTY; without even the implied warranty of
       *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       *  GNU General Public License for more details.
       *
       *  You should have received a copy of the GNU General Public License
       *  along with this program; if not, write to the Free Software
       *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
       *  USA.
       */ 

#include <stdlib.h>
#include <stdio.h>
#include <string>
#include "cross.h"
#ifdef __WIN32__
#include <windows.h>
#endif

#define bit_HTT (1 << 28)
unsigned int HTTSupported(void)
{
  	unsigned int eax, ebx, ecx, edx;
  	cpuid (1, eax, ebx, ecx, edx);
  	return !!(edx & bit_HTT);
}

#define NUM_LOGICAL_BITS 0x00FF0000
unsigned char MaxLPPerPackage(void)
{
  	unsigned int eax, ebx, ecx, edx;
    	if (!HTTSupported()) return 1;
    	cpuid (1, eax, ebx, ecx, edx);
    	return (unsigned char) ((ebx & NUM_LOGICAL_BITS) >> 16);
}

unsigned MaxCoresPerPackage(void)
{						
   	if (!HTTSupported()) return (unsigned char) 1;
	unsigned int eax, ebx, ecx, edx;
	cpuid (0, eax, ebx, ecx, edx);  
	if (eax>3) //max_level>3
   	{
		
		ecx=0;
		__asm__ ("movl    $0,%ecx");
    		cpuid (4, eax, ebx, ecx, edx);   
        	return (unsigned ) ((eax >> 26) +1);
   	}// must be a single-core processor
   	else return 1;
}

#define INITIAL_APIC_ID_BITS 0xFF000000 
unsigned char GetInitAPIC_ID (void)
{
	unsigned int eax, ebx, ecx, edx;
	cpuid (1, eax, ebx, ecx, edx);  
  	return (unsigned char) ((ebx & INITIAL_APIC_ID_BITS) >> 24);
}


void host_detect_multi_cpu ()
{
/*	printf("\n------=[Multi-CPU information]=---------------------------------------\n");
	if (HTTSupported()) printf("HTT supported\n");
	else printf("HTT not supported\n");
	printf("Logical CPUs per package: %d\n",MaxLPPerPackage());
	printf("Cores per package	: %d\n",MaxCoresPerPackage());*/
	long num_procs;
	long procs_online;

	#ifdef __WIN32__
    	SYSTEM_INFO info;
	GetSystemInfo (&info);			//how to detect?!?
	procs_online = num_procs = (unsigned char) info.dwNumberOfProcessors;	  
	#else	
	num_procs = sysconf (_SC_NPROCESSORS_CONF);
	procs_online = sysconf (_SC_NPROCESSORS_ONLN);
	#endif

	printf ("\n  %ld CPU%s installed, %ld online\n", num_procs,
		(num_procs > 1) ? "s" : "", procs_online);

	printf("  Running on CPU with APIC ID  : %d\n",GetInitAPIC_ID());

}

